import datetime

from django.http import HttpResponseRedirect
from django.urls import reverse
from django.views.generic.base import TemplateView

from emissions.models import Diffusion, Emission, Episode, NewsItem
from combo.data.models import Page

import panikweb.views


class Home(panikweb.views.Home):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['emissions'] = Emission.objects.filter(archived=False).order_by('title')
        context['newsitems'] = NewsItem.objects.exclude(date__gt=datetime.date.today()
                ).exclude(expiration_date__lt=datetime.date.today()).order_by('-date')[:3]
        context['extra_pages'] = Page.objects.filter(exclude_from_navigation=False)
        context['emissions'] = []
        return context

    def get(request, *args, **kwargs):
        emissions = Emission.objects.filter(archived=False)
        if False and len(emissions) == 1:
            return HttpResponseRedirect(reverse('emission-view', kwargs={'slug': emissions[0].slug}))
        return super().get(request, *args, **kwargs)


home = Home.as_view()


class EmissionDetailView(panikweb.views.EmissionDetailView):
    def get_emission_context(self, emission, episode_ids=None):
        context = super().get_emission_context(emission, episode_ids)

        if emission.archived:
            current_year = emission.creation_timestamp.replace(month=5, day=1)
        else:
            current_year = datetime.datetime.now().replace(month=5, day=1).replace(year=2019)

        episodes_queryset = Episode.objects.select_related()
        if episode_ids is not None:
            episodes_queryset = episodes_queryset.filter(id__in=episode_ids)
        else:
            episodes_queryset = episodes_queryset.filter(emission=emission)

        context['all_episodes'] = episodes_queryset.filter(
                creation_timestamp__gt=current_year
                ).extra(select={
                        'first_diffusion': 'emissions_diffusion.datetime',
                        },
                        select_params=(False, True),
                        where=['''datetime = (SELECT MIN(datetime)
                                                FROM emissions_diffusion
                                               WHERE episode_id = emissions_episode.id
                                             )'''],
                        tables=['emissions_diffusion'],
                    ).order_by('first_diffusion').distinct()

        return context


emission = EmissionDetailView.as_view()


class ArchivesView(TemplateView):
    template_name = 'archives.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['diffusions'] = (
            Diffusion.objects.filter(episode__soundfile__isnull=False).distinct().order_by('-datetime')
        )
        return context


archives = ArchivesView.as_view()
